package cn.com.qmth.examcloud.ws.starter.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.WithoutStackTrace;
import cn.com.qmth.examcloud.ws.starter.core.MessageOut;
import cn.com.qmth.examcloud.ws.starter.core.SessionInfo;
import cn.com.qmth.examcloud.ws.starter.core.WebSocketHelper;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.*;

import javax.websocket.Session;

@Api(tags = "WebSocket服务")
@RestController
@RequestMapping("api/ctr/ws")
public class WebSocketController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(WebSocketController.class);

    @WithoutStackTrace
    @ApiOperation(value = "API发送消息接口")
    @PostMapping("{path}")
    public String post(@PathVariable String path, @RequestBody String message) {
        User accessUser = getAccessUser();
        String userKey = accessUser.getKey();

        Session session = WebSocketHelper.getSession(path, userKey);
        if (null == session) {
            log.error("NO_WS_SESSION user={} path={}", userKey, path);
            throw new StatusException("100001", "WS会话不存在");
        }

        SessionInfo sessionInfo = WebSocketHelper.getSessionInfo(session);
        if (null == sessionInfo) {
            log.error("NO_SESSION_INFO user={} path={}", userKey, path);
            throw new StatusException("100002", "WS信息不存在");
        }

        MessageOut out = new MessageOut(path, sessionInfo.getSessionId());

        JsonElement jsonEl = null;
        if (StringUtils.isNotBlank(message)) {
            try {
                jsonEl = JsonParser.parseString(message);
            } catch (JsonSyntaxException e) {
                log.error("WS_MESSAGE_INVALID user={} path={}", userKey, path);
                throw new StatusException("100003", "WS信息格式有误");
            }
        }

        out.setContent(jsonEl);

        try {
            ThreadContext.put("TRACE_ID", sessionInfo.getSessionId());
            WebSocketHelper.sendText(session, path, out);
        } finally {
            ThreadContext.put("TRACE_ID", ThreadLocalUtil.getTraceId());
        }

        return JsonUtil.toJson(out);
    }

}
